#ifndef uMainH
#define uMainH

#include <Classes.hpp>
#include <Controls.hpp>
#include <StdCtrls.hpp>
#include <Forms.hpp>
#include <ExtCtrls.hpp>
#include <Menus.hpp>
#include <ComCtrls.hpp>
#include <Graphics.hpp>
#include <jpeg.hpp>
#include <Dialogs.hpp>
#include <ExtDlgs.hpp>

typedef enum
{
  CARD_ABSENT = -1,
  CARD_UNKNOWN = 0,
  CARD_MIFARE_1K,
  CARD_MIFARE_4K,
  CARD_MIFARE_UL,
  CARD_DESFIRE,
  CARD_JAYCOS
} card_type_e;

/*
 * This structure stores everything we need to know regarding
 * the activated card and its data (if some)
 */
typedef struct
{
  /* Card */
  /* ---- */

  bool is_type_a;         /* An ISO 14443-A card has been activated */
  bool is_type_b;         /* An ISO 14443-B card has been activated */
  bool is_tcl;            /* The card has entered ISO 14443-4       */

  card_type_e is_type;    /* The type of the card                   */

  /* ISO 14443-3 parameters of the card */
  union
  {
    /* For a type A card : */
    struct
    {
      unsigned char atq[2];
      unsigned char snr[12];
      unsigned char snrlen;
      unsigned char sak[1];
    } a;
    /* For a type B card : */
    struct
    {
      unsigned char atq[11];
    } b;
  } iso;

  /* ISO 14443-4 parameters of the card */
  struct
  {
    unsigned char ats[32];
    unsigned char atslen;
  } tcl;

  /* Data */
  /* ---- */

  bool has_user;                   /* User data have been found                          */

  unsigned char user_id[8];        /* Identifier of the user                             */
  AnsiString last_name;            /* Last name of the user                              */
  AnsiString first_name;           /* First name of the user                             */
  AnsiString birth_date;           /* Birth date of the user                             */

  signed long purse_value;         /* Value of the e-purse counter                       */

  unsigned short picture_size;     /* A picture has been found in the card               */
  AnsiString picture_file_name;    /* Name of the file where the picture can be found    */

  unsigned char finger_type;       /* Actual type of fingerprint templates               */
  unsigned char finger_size[2];    /* Actual size of fingerprint template (<= 170)       */
  unsigned char finger_data[2][170];
                                   /* Actual fingerprint template (compressed minutiae)  */

} card_data_st;


class TfMain : public TForm
{
__published:
  TMainMenu *mMain;
  TMenuItem *miFile;
  TMenuItem *miQuit;
  TMenuItem *miCard;
  TMenuItem *miRead;
  TMenuItem *miNew;
  TMenuItem *miWrite;
  TMenuItem *N2;
  TMenuItem *miErase;
  TMenuItem *N3;
  TMenuItem *miEdit;
  TGroupBox *gbPurse;
  TLabeledEdit *ePurse;
  TGroupBox *gbPicture;
  TImage *imgPicture;
  TButton *btnPictureBrowse;
  TGroupBox *gbFingerprint;
  TButton *btnPictureRemove;
  TLabel *lbFingerprint;
  TButton *btnFingerEnroll;
  TButton *btnFingerRemove;
  TButton *btnFingerVerify;
  TPanel *pTop;
  TLabel *lbTitle;
  TLabel *lbCaption;
  TImage *imgLogo;
  TStatusBar *status;
  TImage *img14443;
  TTimer *tmrStart;
  TGroupBox *gbTypeA;
  TLabeledEdit *eUid;
  TLabel *lbTypeA;
  TLabeledEdit *eAtsA;
  TGroupBox *gbTypeB;
  TLabel *lbTypeB;
  TLabeledEdit *ePupi;
  TLabeledEdit *eAtsB;
  TGroupBox *gbNoCard;
  TOpenPictureDialog *dlgOpenPicture;
  TPanel *pIdentity;
  TLabeledEdit *eUserID;
  TLabeledEdit *eFirstName;
  TLabeledEdit *eLastName;
  TLabeledEdit *eBirthDate;
  TLabel *lbFingerOK;
  TLabel *lbFingerKO;
  TLabel *Label1;
        TImage *imgHeader;
  void __fastcall FormShow(TObject *Sender);
  void __fastcall EraseFields();
  void __fastcall ErasePicture();
  void __fastcall DataToDisplay();
  void __fastcall SetEditable(bool editable);
  void __fastcall miNewClick(TObject *Sender);
  void __fastcall miEditClick(TObject *Sender);
  void __fastcall miWriteClick(TObject *Sender);
  void __fastcall miEraseClick(TObject *Sender);
  void __fastcall miReadClick(TObject *Sender);
  void __fastcall FormCreate(TObject *Sender);
  void __fastcall btnPictureBrowseClick(TObject *Sender);
  void __fastcall btnFingerEnrollClick(TObject *Sender);
  void __fastcall btnFingerVerifyClick(TObject *Sender);
  void __fastcall btnFingerRemoveClick(TObject *Sender);
  void __fastcall miQuitClick(TObject *Sender);
  void __fastcall btnPictureRemoveClick(TObject *Sender);
private:
  int after_find_do_task;
public:		// Dclarations de l'utilisateur
  __fastcall TfMain(TComponent* Owner);

  void __fastcall OnWorkerDone(int WorkerAction, bool WorkerResult);
  card_data_st card_data;
  AnsiString temp_picture_file_name;
};

extern PACKAGE TfMain *fMain;

#endif
