//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "uTray.h"
#include "uSettings.h"

#include "products/springprox/springprox.h"
#include <stdio.h>

/*
 * SendInputChar
 * -------------
 * Emulate a keyboard key
 */
void SendInputChar(char chr)
{
  /* Translations */
  if ((chr >= 'a') && (chr <= 'z'))
  {
    /* Lower case letter translated to upper case */
    chr = (char) ((chr + 0x41) - 'a');
  } else if ((chr >= '0') && (chr <= '9'))
  {
    /* Digits on the numeric pad */
    chr = (char) ((chr + 0x60) - '0');
  } else
  {
    switch (chr)
    {
        /* A few special chars */
      case '&':
        chr = 0x31;
        break;
      case '':
        chr = 0x32;
        break;
      case '"':
        chr = 0x33;
        break;
      case '\'':
        chr = 0x34;
        break;
      case '(':
        chr = 0x35;
        break;
      case '-':
        chr = 0x36;
        break;
      case '':
        chr = 0x37;
        break;
      case '_':
        chr = 0x38;
        break;
      case '':
        chr = 0x39;
        break;
      case '':
        chr = 0x30;
        break;
      case '.':
        chr = 0x6E;
        break;
    }
  }


  /* Send the keybord sequence */
  keybd_event(chr, 0, 0, 0);
  keybd_event(chr, 0, KEYEVENTF_KEYUP, 0);
}

/*
 * SendInputString
 * ---------------
 * Emulate a keyboard key sequence
 */
void SendInputString(char *str)
{
  while (*str)
  {
    SendInputChar(*str);
    str++;
  }
}


#pragma package(smart_init)
#pragma link "trayicon"
#pragma resource "*.dfm"
TfTray *fTray;

__fastcall TfTray::TfTray(TComponent * Owner):TForm(Owner)
{
}

void __fastcall TfTray::miQuitClick(TObject * Sender)
{
  Application->Terminate();
}

void __fastcall TfTray::miPauseClick(TObject * Sender)
{
  CanRun = false;
  miPause->Enabled = false;
  miResume->Enabled = true;
}


void __fastcall TfTray::miResumeClick(TObject * Sender)
{
  CanRun = true;
  miResume->Enabled = false;
  miPause->Enabled = true;
}

void __fastcall TfTray::tmrWorkTimer(TObject * Sender)
{
  short rc;

  unsigned char a_atq[2];
  unsigned char a_uid[12];
  unsigned char a_uidlen;
  unsigned char a_sak[1];
  unsigned char b_atq[11];
  unsigned char mif_key[6];
  unsigned char mif_data[16];
  AnsiString TagIdent = "";
  TDateTime When;
  __int64 WantMs;

  static bool SettingsOK = false;
  static AnsiString PreviousTagIdent = "";
  static int TicksBeforeNextTag = 0;

  /* Remember when we've entered the function */
  When = Now();

  if (!CanRun)
  {
    tmrWork->Interval  = 500;
    TrayIcon->Hint = "Pause";
    TicksBeforeNextTag = 0;
    SettingsOK = false;
    return;
  }

  /* Try to acquire the mutex. Failture means that the configuration applet has been launched */
  if (ConfigMutex != NULL)
  {
    DWORD res;

    res = WaitForSingleObject(ConfigMutex, 0);
    if (res == WAIT_TIMEOUT)
    {
      /* The configuration applet is running... */
      TrayIcon->Hint = "Configuration applet is running";
      tmrWork->Interval  = 1000;
      TicksBeforeNextTag = 0;
      SettingsOK = false;
      return;
    }
  }

  /* (re)load settings if needed */
  if (!SettingsOK)
  {
    Settings->LoadSettings();
    SettingsOK = true;
  }

  /* Stop the timer during (maybe long) reader sequence */
  tmrWork->Enabled = false;

  if (CsbActive)
  {
    do
    {
      /* We must wait the specified delay between two tags */
      if (TicksBeforeNextTag > 0)
      {
        TicksBeforeNextTag--;
        if (TicksBeforeNextTag > 0)
          break;
      }

      do
      {
        if ((Settings->ScanMode == 0) || (Settings->ScanMode == 2))
        {
          /* ISO 14443-B part */
          /* ---------------- */

          rc = SPROX_B_SelectAny(0, b_atq);
          if (rc == MI_OK)
          {
            /* Process 14443-B information */
            SPROX_ControlLedY(0, 1, 0);
            TagIdent = Settings->Format14443B(b_atq);
            break;
          }
          if (rc < -128)
            break;
        }

        if ((Settings->ScanMode == 0) || (Settings->ScanMode == 1)
            || (Settings->ScanMode == 3))
        {
          /* ISO 14443-A part */
          /* ---------------- */

          a_uidlen = sizeof(a_uid);
          rc = SPROX_A_SelectAny(a_atq, a_uid, &a_uidlen, a_sak);
          if (rc == MI_OK)
          {
            if (Settings->ScanMode == 3)
            {
              /* Check that this is a Mifare */
              /* --------------------------- */

              if ((a_atq[1] == 0x00)
                  && ((a_atq[0] == 0x02) || (a_atq[0] == 0x04)))
              {
                if (Settings->GetMifareKey(mif_key))
                  rc =
                    SPROX_MifStReadBlock(NULL, Settings->MifareBlock,
                                         mif_data, mif_key);
                else
                  rc =
                    SPROX_MifStReadBlock(NULL, Settings->MifareBlock,
                                         mif_data, NULL);
                if (rc == MI_OK)
                {
                  /* Read successfull */
                  SPROX_ControlLedY(0, 1, 0);
                  TagIdent = Settings->FormatMifare(mif_data);
                  break;
                }
                if (rc < -128)
                  break;
                /* Read failed */
                SPROX_ControlLedY(1, 0, 0);
              }
            } else
            {
              /* Process 14443-A information */
              SPROX_ControlLedY(0, 1, 0);
              TagIdent =
                Settings->Format14443A(a_atq, a_uid, a_uidlen, a_sak);
              break;
            }
          }
          if (rc < -128)
            break;
        }
      } while (0);

      if (rc < -128)
      {
        /* Fatal error */
        CsbActive = false;
        SPROX_ReaderClose();
        TrayIcon->Hint = "Reader error";
      }

      if (TagIdent != "")
      {
        /* Now filter the data */
        /* ------------------- */

        if (PreviousTagIdent == TagIdent)
        {
          /* This is the same tag as earlier */
          if ((Settings->ScanFilter == 0) || (Settings->ScanFilter == 2))
            TagIdent = "";      /* Do not repeat (never repeat, or wait until tag removed) */

        } else
        {
          /* New tag */
          PreviousTagIdent = TagIdent;
        }

        if (TagIdent != "")
        {
          /* Init filter */
          TicksBeforeNextTag = Settings->ScanDelay;

          /* Send the actual keyboard sequence */
          if (Settings->Prefix == "<ENTER>") SendInputString("\r\n"); else
          if (Settings->Prefix == "<TAB>") SendInputString("\t");

          SendInputString(TagIdent.c_str());

          if (Settings->Postfix == "<ENTER>") SendInputString("\r\n"); else
          if (Settings->Postfix == "<TAB>") SendInputString("\t");

          /* Beep */
          if (Settings->ScanBeep)
            MessageBeep(0xFFFFFFFF);
        }

        TrayIcon->Hint = TagIdent;

      } else
      {
        if (Settings->ScanFilter == 2)
          PreviousTagIdent = "";  /* Tag removed, forget it now */

        TrayIcon->Hint = "Waiting for a tag";
        SPROX_ControlLedY(0, 0, 0);
      }

    } while (0);

    WantMs = MilliSecondsBetween(When, Now());
    if ((WantMs < 100) && (WantMs > 25))
      tmrWork->Interval = WantMs;
    else
      tmrWork->Interval = 100;

    while (WantMs > 150)
    {
      if (TicksBeforeNextTag > 0) TicksBeforeNextTag--;
      WantMs -= 100;
    }

  } else
  {
    TicksBeforeNextTag = 0;
    tmrWork->Interval  = 500;

    if (SPROX_ReaderOpen(Settings->Device.c_str()) == MI_OK)
    {
      CsbActive = true;
      SPROX_ControlLedY(0, 0, 0);
      SPROX_ControlRF(0);
      TrayIcon->Hint = "Waiting for a tag";
    } else
    {
      TrayIcon->Hint = "Reader not found";
    }
  }

  /* Release the mutex */
  if (ConfigMutex != NULL)
    ReleaseMutex(ConfigMutex);

  tmrWork->Enabled = true;
}

void __fastcall TfTray::FormCreate(TObject * Sender)
{
  ConfigMutex = NULL;
  TrayMutex = NULL;

  /* The TrayMutex is necessary to ensure that only one instance of this application is running */
  TrayMutex = CreateMutex(NULL, true, "SPROX_ScanUid");
  if ((TrayMutex == NULL) || (GetLastError() == ERROR_ALREADY_EXISTS))
  {
    /* Another instance is already there */
    if (TrayMutex != NULL)
    {
      ReleaseMutex(TrayMutex);
      CloseHandle(TrayMutex);
      TrayMutex = NULL;
    }
    /* Let's commit suicide */
    MessageBox(0, "Another instance is already running, exiting", Caption.c_str(), 0);
    Application->Terminate();
    return;
  }

  /* The ConfigMutex is used by the configuration applet */
  ConfigMutex = CreateMutex(NULL, false, "SPROX_ScanUid_Cfg");
  /* We don't care for the result... */

  CsbActive = false;
  CanRun = true;
  Settings = new TSettings();
}

//---------------------------------------------------------------------------

void __fastcall TfTray::FormDestroy(TObject *Sender)
{
  if (CsbActive)
    SPROX_ReaderClose();

  if (TrayMutex != NULL)
  {
    ReleaseMutex(TrayMutex);
    CloseHandle(TrayMutex);
    TrayMutex = NULL;
  }
  if (ConfigMutex != NULL)
  {
    ReleaseMutex(ConfigMutex);
    CloseHandle(ConfigMutex);
    ConfigMutex = NULL;
  }
}
//---------------------------------------------------------------------------

